/*
 *  An implementation of the ARC4 algorithm
 *  from http://www.cr0.net:8040/code/crypto/
 *
 *  Copyright (C) 2001-2003  Christophe Devine
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "rc4.h"

void rc4_setup( struct rc4_state *s, unsigned char *key,  int length )
{
    int i, j, k, *m, a;

    s->x = 0;
    s->y = 0;
    m = s->m;

    for( i = 0; i < 256; i++ )
    {
        m[i] = i;
    }

    j = k = 0;

    for( i = 0; i < 256; i++ )
    {
        a = m[i];
        j = (unsigned char) ( j + a + key[k] );
        m[i] = m[j]; m[j] = a;
        if( ++k >= length ) k = 0;
    }
}

void rc4_crypt( struct rc4_state *s, unsigned char *data, int length )
{ 
    int i, x, y, *m, a, b;

    x = s->x;
    y = s->y;
    m = s->m;

    for( i = 0; i < length; i++ )
    {
        x = (unsigned char) ( x + 1 ); a = m[x];
        y = (unsigned char) ( y + a );
        m[x] = b = m[y];
        m[y] = a;
        data[i] ^= m[(unsigned char) ( a + b )];
    }

    s->x = x;
    s->y = y;
}


/*
 *  An implementation of the ARC4 algorithm
 *  from http://www.cr0.net:8040/code/crypto/
 *  Copyright (C) 2001-2003  Christophe Devine
        A few comments regarding these cryptographic primitives: 
        
        Collisions [1] have been found in MD5 by Xiaoyun Wang et al. It is highly recomended to use stronger cryptographic hash functions, such as SHA-1 which is the current de-facto standard.
          
        RC4 is considered cryptographically secure, provided that three hygiene rules are followed:
          
        The first 512 bytes of RC4's output must be dropped [2] in order to defeat weak-keys attacks [3], and other attacks related to the RC4 key schedule.
          
        Because RC4 is used as a one-time-pad, one should never use the same key twice. Most protocols use a random IV (initialization vector) appended to the secret key. Then a hash function (say, SHA-1) should be applied, so as to defeat related-keys attacks [4].
          
        No more than 2^30.6 bytes should be encrypted with RC4 (whatever the key), as it suffers for a distinguisher attack [5]. To encrypt larger data streams, one should preferably use AES in counter mode (AES-CTR), which acts like a stream cipher.
          
        It is strongly recommended to include a message authentication code, for example HMAC [6], using preferably an encrypt-then-mac scheme [7]. 
        
        [1] Xiaoyun Wang, Dengguo Feng, Xuejia Lai, Hongbo Yu, "Collisions for Hash Functions MD4, MD5, HAVAL-128 and RIPEMD".
          
        http://eprint.iacr.org/2004/199/
          
        [2] Ilya Mironov, "(Not So) Random Shuffles of RC4", in Proceedings of CRYPTO 2002.
          
        http://citeseer.ist.psu.edu/531224.html
          
        [3] Andrew Roos, "A Class of Weak Keys in the RC4 Stream Cipher", in sci.crypt.research, 1995-09-22.
          
        http://groups-beta.google.com/group/sci.crypt.research/msg/078aa9249d76eacc?dmode=source
          
        [4] Scott Fluhrer, Itsik Mantin, Adi Shamir, "Weaknesses in the Key Scheduling Algorithm of RC4", in Proceedings of SAC 2001.
          
        http://citeseer.ist.psu.edu/fluhrer01weaknesses.html
          
        [5] Scott Fluhrer, David McGrew, "Statistical Analysis of the Alleged RC4 Keystream Generator", in Proceedings of FSE 2000.
          
        http://www.mindspring.com/~dmcgrew/rc4-03.pdf
          
        [6] Mihir Bellare, Ran Canetti, Hugo Krawczyk, "Message Authentication Using Hash Functions: the HMAC Construction", in Proceedings of CRYPTO 1996.
          
        http://citeseer.ist.psu.edu/bellare96message.html
          
        [7] Hugo Krawczyk, "The Order of Encryption and Authentication for Protecting Communications (or: How Secure Is SSL?)", in Proceedings of CRYPTO 2001.
          
http://citeseer.ist.psu.edu/krawczyk01order.html

 */